#ifndef NETSERVICE_H
#define NETSERVICE_H

#include "nethost.h"

class NetTask
{
public:
  enum Result {
    READY,ERROR,RETRY,KEEP
  };
  NetTask() : m_stop(false) {}
  virtual ~NetTask() {}
  
  virtual Result run(NetHost &host,QString &resultEvent) = 0;
  
  bool isStopped() const { return m_stop; }
  void doStop() { m_stop = true; }
  
protected:
  bool m_stop;
};

class NetService : public QThread
{
  Q_OBJECT
  
public:
  NetService(const AddrPair &addrPair,const QString &name);
  ~NetService();
  
  //! return service name
  QString name() const { return m_name; }
  
  //! add a task (force run flushes all old tasks)
  void addTask(NetTask *task,bool cancelAll=false);
  
  //! tell the service to stop and cancel all scheduled tasks
  void stop();
  
  //! reset service (if service failed fatal)
  void reset();
  
signals:
  //! emit a message
  void networkEvent(const QString &);
  //! sent packet
  void sentPacket(int size);
  //! received packet
  void receivedPacket(int size);
  
protected:
  AddrPair m_addrPair;
  QString  m_name;
  
  QMutex m_mutex;
  bool m_doStop;
  bool m_failedFatal;
  QQueue<NetTask *> m_tasks;

  //! thread execution
  void run();
  void flushTasks();
  void flushPackets(NetHost *host);
  void snooze(NetHost *host);
};

#endif
